/// <reference path="../tsDefinitions/phaser.d.ts" />
/// <reference path="./DataManager.ts" />

module GGJ16 {
    
    
    export class Inventory 
    {
        g: Phaser.Game;
        arrayItems: Array<Phaser.Sprite>;
        
        constructor(g: Phaser.Game) 
        {
            this.g = g;
            this.arrayItems = new Array<Phaser.Sprite>();
            
            this.fillInventory();
            this.drawInventory();
        }
        
        fillInventory()
        {
            //Ruby
            if(DataManager.getInstance().ruby)
                this.arrayItems.push(new Phaser.Sprite(this.g, 0, 0, 'ruby'));
                
            //Key
            if(DataManager.getInstance().keyDrawer)
                this.arrayItems.push(new Phaser.Sprite(this.g, 0, 0, 'key'));
                
            //Piece1
            if(DataManager.getInstance().piece1)
                this.arrayItems.push(new Phaser.Sprite(this.g, 0, 0, 'piece1puzzle64'));
                
            //Piece2
            if(DataManager.getInstance().piece2)
                this.arrayItems.push(new Phaser.Sprite(this.g, 0, 0, 'piece2puzzle64'));
                
            //Piece3
            if(DataManager.getInstance().piece3)
                this.arrayItems.push(new Phaser.Sprite(this.g, 0, 0, 'piece3puzzle64'));
                
            //VialY
            if(DataManager.getInstance().vialY)
                this.arrayItems.push(new Phaser.Sprite(this.g, 0, 0, 'vialYellowFull'));
                
            //VialC
            if(DataManager.getInstance().vialC)
                this.arrayItems.push(new Phaser.Sprite(this.g, 0, 0, 'vialCyanFull'));
                
            //VialBr
            if(DataManager.getInstance().vialM)
                this.arrayItems.push(new Phaser.Sprite(this.g, 0, 0, 'vialMagentaFull'));
                
            //Hint
            if(DataManager.getInstance().note)
                this.arrayItems.push(new Phaser.Sprite(this.g, 0, 0, 'note'));
                
                this.arrayItems.forEach(element => {
                    element.inputEnabled = true;
                    element.events.onInputDown.add(this.clickedInventoryItem, this);                
            });
        }
        
        clickedInventoryItem()
        {
            new Monologue(this.g, "< You don't need to click an item in order to use it >");
        }

        drawInventory()
        {
            var count = 1;
            var offsetX = 40;
            var offsetY = 10;
            this.arrayItems.forEach(element => {
                element.position.set(offsetX + (element.width*count), offsetY + element.height/2);
                element.anchor.set(0.5);
                this.g.add.existing(element); 
                count++;
            });
        }
        
        tick()
        {
            
            this.arrayItems.forEach(element => {
                
                element.destroy();
            });
            
            this.arrayItems = new Array<Phaser.Sprite>();
            
            this.fillInventory();
            this.drawInventory();
        }
    }
}