/// <reference path="../tsDefinitions/phaser.d.ts" />

module GGJ16 {
    
    export class Puzzle5Piece extends Phaser.Sprite {

        xidx:number;
        yidx:number;
        p5: Puzzle5;
        slideTime:number;
        
        constructor(p5: Puzzle5, identifier:string, x:number, y:number, xidx:number, yidx:number) {
            
            super(p5.game, x, y, identifier);
            this.alpha = 0.7
            this.anchor.set(0.5);
            p5.game.add.existing(this);
            this.xidx = xidx;
            this.yidx = yidx;
            this.inputEnabled = true;
            this.p5 = p5;
            this.events.onInputDown.add(this.handleClick, this);
            this.events.onInputOver.add(this.handleOver, this);
            this.events.onInputOut.add(this.handleOut, this);
            this.slideTime = 280;
            
        }
        
        iAmHere(xidx:number, yidx:number): boolean {
            
            if( Number(this.xidx) === Number(xidx) && Number(this.yidx) == Number(yidx) )
                return true;
            
            return false;
            
        }
        
        handleOver()
        {
            if(DataManager.getInstance().piece1Placed && DataManager.getInstance().piece2Placed && DataManager.getInstance().piece3Placed)
            this.alpha = 1.0;
        }
        
        handleOut()
        {
            if(DataManager.getInstance().piece1Placed && DataManager.getInstance().piece2Placed && DataManager.getInstance().piece3Placed)
                this.alpha = 0.7;
        }
        
        handleClick() {
            
            if( this.p5.minigameCanRun && !this.p5.pieceMoving ) {
            
                var pieces = this.p5.pieces;
                var canMove = true;
                var nx = Number(this.xidx);
                var ny = Number(this.yidx);
                
                if( nx - 1 >= 0 ) {
                    
                    pieces.forEach( piece => {
                        if( piece.iAmHere(nx - 1, ny) )
                            canMove = false;
                    });
                    
                    if ( canMove ) {
                        this.p5.pieceMoving = true;
                        this.p5.game.add.tween(this).to( { x: (this.x - this.p5.numPieceDimensions) }, this.slideTime, Phaser.Easing.Linear.None, true).onComplete.add(this.releaseGame, this);
                        this.xidx = Number(this.xidx) - 1;
                        this.p5.sfxMovePiece.play();
                        return;
                    }
                }
                
                canMove = true
                if( nx + 1 <= 2 ) {
                    
                    pieces.forEach( piece => {
                        if( piece.iAmHere(nx + 1, ny) )
                            canMove = false;
                    });
                    
                    if ( canMove ) {
                        this.p5.pieceMoving = true;
                        this.p5.game.add.tween(this).to( { x: (this.x + this.p5.numPieceDimensions) }, this.slideTime, Phaser.Easing.Linear.None, true).onComplete.add(this.releaseGame, this);
                        this.xidx = Number(this.xidx) + 1;
                        this.p5.sfxMovePiece.play();
                        return;
                    }
                }
                
                canMove = true
                if( ny - 1 >= 0 ) {
                    
                    pieces.forEach( piece => {
                        if( piece.iAmHere(nx, ny - 1) )
                            canMove = false;
                    });
                    
                    if ( canMove ) {
                        this.p5.pieceMoving = true;
                        this.p5.game.add.tween(this).to( { y: (this.y - this.p5.numPieceDimensions) }, this.slideTime, Phaser.Easing.Linear.None, true).onComplete.add(this.releaseGame, this);
                        this.yidx = Number(this.yidx) - 1;
                        this.p5.sfxMovePiece.play();
                        return;
                    }
                }
                
                canMove = true
                if( ny + 1 <= 2 ) {
                    
                    pieces.forEach( piece => {
                        if( piece.iAmHere(nx, ny + 1) )
                            canMove = false;
                    });
                    
                    if ( canMove ) {
                        this.p5.pieceMoving = true;
                        this.p5.game.add.tween(this).to( { y: (this.y + this.p5.numPieceDimensions) }, this.slideTime, Phaser.Easing.Linear.None, true).onComplete.add(this.releaseGame, this);
                        this.yidx = Number(this.yidx) + 1;
                        this.p5.sfxMovePiece.play();
                        return;
                    }
                }
            }
        }
        
        releaseGame() {
            this.p5.pieceMoving = false;
        }
    }
}