/// <reference path="../tsDefinitions/phaser.d.ts" />
/// <reference path="./DataManager.ts" />

module GGJ16 {
    export class Vial extends Phaser.Sprite {
        g: Phaser.Game;
        emitter: Phaser.Particles.Arcade.Emitter;
        colorState: vialState;
        id: number;
        animating: boolean;
        
        constructor(g: Phaser.Game, x: number, y: number, type: vialState, id: number) 
        {
            this.animating = false;
            this.colorState = type;
            this.id = id;
            switch(type)
            {
                case vialState.red:
                    if(DataManager.getInstance().vialRStatus == vialState.red){
                        super(g, x, y, 'vialRedFull');
                    }else if(DataManager.getInstance().vialRStatus == vialState.hRed){
                        super(g, x, y, 'vialRedHalf');
                        this.colorState = DataManager.getInstance().vialRStatus;
                    }
                    else{
                        super(g, x, y, 'vialEmpty');
                        this.colorState = vialState.emptyUnusable;
                    }
                    break;
                case vialState.green:
                    if(DataManager.getInstance().vialGStatus == vialState.green){
                        super(g, x, y, 'vialGreenFull');
                    }else if(DataManager.getInstance().vialGStatus == vialState.hGreen){
                        super(g, x, y, 'vialGreenHalf');
                        this.colorState = DataManager.getInstance().vialGStatus;
                    }
                    else{
                        super(g, x, y, 'vialEmpty');
                        this.colorState = vialState.emptyUnusable;
                    }
                    break;
                case vialState.blue:
                    if(DataManager.getInstance().vialBStatus == vialState.blue){
                        super(g, x, y, 'vialBlueFull');
                    }else if(DataManager.getInstance().vialBStatus == vialState.hBlue){
                        super(g, x, y, 'vialBlueHalf');
                        this.colorState = DataManager.getInstance().vialBStatus;
                    }
                    else{
                        super(g, x, y, 'vialEmpty');
                        this.colorState = vialState.emptyUnusable;
                    }
                    break;
                default:
                    super(g, x, y, 'vialEmpty');
                    this.colorState = vialState.emptyUnusable;
                    break;
            }
            this.g = g;
            this.anchor.x = 0.5;
            this.anchor.y = 0.5;
            
            this.inputEnabled = true;
            
            g.add.existing(this);
            // this.emitter = g.game.add.emitter(x + ptm/2, y + ptm/2);
            // this.emitter.makeParticles('ministar', 1, 20, false);
            
            if(this.id === 1 && !DataManager.getInstance().vial1Presence ||
               this.id === 2 && !DataManager.getInstance().vial2Presence ||
               this.id === 3 && !DataManager.getInstance().vial3Presence)
               {
                   this.colorState = vialState.invisible;
                    this.inputEnabled = false;
                    this.alpha = 0.0;
            }
            else if(this.id === 1)
            {
                this.colorState = DataManager.getInstance().vial1Status;
            }
            else if(this.id === 2)
            {
                this.colorState = DataManager.getInstance().vial2Status;
            }
            else if(this.id === 3)
            {
                this.colorState = DataManager.getInstance().vial3Status;
            }
            
            switch(this.colorState)
            {
                case vialState.hRedResult:
                    this.loadTexture('vialRedHalf');
                    break;
                case vialState.hGreenResult:
                    this.loadTexture('vialGreenHalf');
                    break;
                case vialState.hBlueResult:
                    this.loadTexture('vialBlueHalf');
                    break;
            }
            
            console.log("Created vial " + this.id);
            
        }
        
        setNewColor(color: vialState) //this is used for the right vials only...
        {
            switch(color)
            {
                case vialState.red:
                case vialState.hRed:
                    this.loadTexture('vialRedHalf');
                    this.colorState = vialState.hRedResult;
                    // DataManager.getInstance().vialRStatus = this.colorState;
                    break;
                case vialState.green:
                case vialState.hGreen:
                    this.loadTexture('vialGreenHalf');
                    this.colorState = vialState.hGreenResult;
                    // DataManager.getInstance().vialGStatus = this.colorState;
                    break;
                case vialState.blue:
                case vialState.hBlue:
                    this.loadTexture('vialBlueHalf');
                    this.colorState = vialState.hBlueResult;
                    // DataManager.getInstance().vialBStatus = this.colorState;
                    break;
            }
                    
            switch(this.id) //update DataManager
            {
                case 1:    
                    DataManager.getInstance().vial1Status = this.colorState;
                    break;
                case 2:    
                    DataManager.getInstance().vial2Status = this.colorState;
                    break;
                case 3:    
                    DataManager.getInstance().vial3Status = this.colorState;
                    break;
            }
        }
        
        mixColor(color: vialState):boolean //this is for the right vials
        {
            switch(color) //color that is coming
            {
                case vialState.red:
                case vialState.hRed:
                    switch(this.colorState)
                    {
                        case vialState.hGreenResult:
                            if(DataManager.getInstance().vialY)
                            {
                                new Monologue(this.game, "I already got an Yellow Vial...");
                                return false;
                            }
                            this.loadTexture('vialYellowFull');
                            this.colorState = vialState.yellow;
                            
                            new Monologue(this.game, "I got an Yellow Vial...");
                            DataManager.getInstance().canClick = false;
                            var tween = this.game.add.tween(this).to( { x: [ 0, this.width + 10 ], y: [ this.position.y, 10] }, 500, "Sine.easeInOut", true, 0, 0, false);
                            tween.interpolation(Phaser.Math.bezierInterpolation);
                            tween.onComplete.add( () => {
                                DataManager.getInstance().canClick = true;
                                DataManager.getInstance().vialY = true;
                                this.alpha = 0.0;
                            });
                            
                            // DataManager.getInstance().vialY = true;
                            break;
                            
                        case vialState.hBlueResult:
                            if(DataManager.getInstance().vialM)
                            {
                                new Monologue(this.game, "I already got a Magenta Vial...");
                                return false;
                            }
                            this.loadTexture('vialMagentaFull');
                            this.colorState = vialState.magenta;
                            // DataManager.getInstance().vialM = true;
                            new Monologue(this.game, "I got a Magenta Vial...");
                            DataManager.getInstance().canClick = false;
                            var tween = this.game.add.tween(this).to( { x: [ 0, this.width + 10 ], y: [ this.position.y, 10] }, 500, "Sine.easeInOut", true, 0, 0, false);
                            tween.interpolation(Phaser.Math.bezierInterpolation);
                            tween.onComplete.add( () => {
                                DataManager.getInstance().canClick = true;
                                DataManager.getInstance().vialM = true;
                                this.alpha = 0.0;
                            });
                            
                            break;
                    }
                    break;
                case vialState.green:
                case vialState.hGreen:
                    switch(this.colorState)
                    {
                        case vialState.hRedResult:
                            if(DataManager.getInstance().vialY)
                            {
                                new Monologue(this.game, "I already got an Yellow Vial...");
                                return false;
                            }
                            this.loadTexture('vialYellowFull');
                            this.colorState = vialState.yellow;
                            // DataManager.getInstance().vialY = true;
                            new Monologue(this.game, "I got an Yellow Vial...");
                            DataManager.getInstance().canClick = false;
                            var tween = this.game.add.tween(this).to( { x: [ 0, this.width + 10 ], y: [ this.position.y, 10] }, 500, "Sine.easeInOut", true, 0, 0, false);
                            tween.interpolation(Phaser.Math.bezierInterpolation);
                            tween.onComplete.add( () => {
                                DataManager.getInstance().canClick = true;
                                DataManager.getInstance().vialY = true;
                                this.alpha = 0.0;
                            });
                            break;
                            
                        case vialState.hBlueResult:
                            if(DataManager.getInstance().vialC)
                            {
                                new Monologue(this.game, "I already got a Cyan Vial...");
                                return false;
                            }
                            this.loadTexture('vialCyanFull');
                            this.colorState = vialState.cyan;
                            // DataManager.getInstance().vialC = true;
                            new Monologue(this.game, "I got a Cyan Vial...");
                            DataManager.getInstance().canClick = false;
                            var tween = this.game.add.tween(this).to( { x: [ 0, this.width + 10 ], y: [ this.position.y, 10] }, 500, "Sine.easeInOut", true, 0, 0, false);
                            tween.interpolation(Phaser.Math.bezierInterpolation);
                            tween.onComplete.add( () => {
                                DataManager.getInstance().canClick = true;
                                DataManager.getInstance().vialC = true;
                                this.alpha = 0.0;
                            });
                            break;
                    }
                    break;
                case vialState.blue:
                case vialState.hBlue:
                    switch(this.colorState)
                    {
                        case vialState.hRedResult:
                            if(DataManager.getInstance().vialM)
                            {
                                new Monologue(this.game, "I already got a Magenta Vial...");
                                return false;
                            }
                            this.loadTexture('vialMagentaFull');
                            this.colorState = vialState.magenta;
                            // DataManager.getInstance().vialM = true;
                            new Monologue(this.game, "I got a Magenta Vial...");
                            DataManager.getInstance().canClick = false;
                            var tween = this.game.add.tween(this).to( { x: [ 0, this.width + 10 ], y: [ this.position.y, 10] }, 500, "Sine.easeInOut", true, 0, 0, false);
                            tween.interpolation(Phaser.Math.bezierInterpolation);
                            tween.onComplete.add( () => {
                                DataManager.getInstance().canClick = true;
                                DataManager.getInstance().vialM = true;
                                this.alpha = 0.0;
                            });
                            
                            break;
                            
                        case vialState.hGreenResult:
                            if(DataManager.getInstance().vialC)
                            {
                                new Monologue(this.game, "I already got a Cyan Vial...");
                                return false;
                            }
                            this.loadTexture('vialCyanFull');
                            this.colorState = vialState.cyan;
                            // DataManager.getInstance().vialC = true;
                            new Monologue(this.game, "I got a Cyan Vial...");
                            DataManager.getInstance().canClick = false;
                            var tween = this.game.add.tween(this).to( { x: [ 0, this.width + 10 ], y: [ this.position.y, 10] }, 500, "Sine.easeInOut", true, 0, 0, false);
                            tween.interpolation(Phaser.Math.bezierInterpolation);
                            tween.onComplete.add( () => {
                                DataManager.getInstance().canClick = true;
                                DataManager.getInstance().vialC = true;
                                this.alpha = 0.0;
                            });
                            break;
                    }
                    break;
            }
            
            switch(this.id)
            {
                case 1:
                    DataManager.getInstance().vial1Presence = false;
                    break;
                case 2:
                    DataManager.getInstance().vial2Presence = false;
                    break;
                case 3:
                    DataManager.getInstance().vial3Presence = false;
                    break;
            }
            
            this.inputEnabled = false;
            // this.alpha = 0.0;
            return true;
        }
        
        useThePaint()
        {
            switch(this.colorState)
            {
                case vialState.red:
                    // this.g.add.tween(this).to( { alpha: 0.0 }, 2000, Phaser.Easing.Linear.None, true, 0, 1000, true);
                    // this.animating = true;
                    // var switchSprite: Phaser.Tween = this.game.add.tween(this.alpha).to(0.0, 500, Phaser.Easing.Linear.None, true);
                    // switchSprite.onComplete.add( () => {
                    //     this.loadTexture('vialRedHalf');
                    //     this.game.add.tween(this.alpha).to(1.0, 500, Phaser.Easing.Linear.None, true);
                    //     this.animating = false;
                    // }, this);
                    
                    this.loadTexture('vialRedHalf');
                    this.colorState = vialState.hRed;
                    DataManager.getInstance().vialRStatus = this.colorState;
                    break;
                case vialState.hRed:
                    this.loadTexture('vialEmpty');
                    this.colorState = vialState.emptyUnusable;
                    DataManager.getInstance().vialRStatus = this.colorState;
                    break;
                case vialState.green:
                    this.loadTexture('vialGreenHalf');
                    this.colorState = vialState.hGreen;
                    DataManager.getInstance().vialGStatus = this.colorState;
                    break;
                case vialState.hGreen:
                    this.loadTexture('vialEmpty');
                    this.colorState = vialState.emptyUnusable;
                    DataManager.getInstance().vialGStatus = this.colorState;
                    break;
                case vialState.blue:
                    this.loadTexture('vialBlueHalf');
                    this.colorState = vialState.hBlue;
                    DataManager.getInstance().vialBStatus = this.colorState;
                    break;
                case vialState.hBlue:
                    this.loadTexture('vialEmpty');
                    this.colorState = vialState.emptyUnusable;
                    DataManager.getInstance().vialBStatus = this.colorState;
                    break;
            }
        }
    }
}