/// <reference path="../../tsDefinitions/phaser.d.ts" />
/// <reference path="../DataManager.ts" />
/// <reference path="../Monologue.ts" />

module GGJ16 {
    
    export class Puzzle3 extends Phaser.State {
        
        //
        // NOTE: Game Attributes
        //
        sfxUnlock: Phaser.Sound;
        sfxSuccess: Phaser.Sound;
        sfxFootsteps: Phaser.Sound;
        jsnLevel: Phaser.Tilemap;
        vialHolder1: Phaser.Sprite;
        vialHolder2: Phaser.Sprite;
        vialHolder3: Phaser.Sprite;
        statue: Phaser.Sprite;
        inventory: Inventory;
        arrowBack: Phaser.Sprite;
        piece1: Phaser.Sprite;
        
        //
        // NOTE: Control
        //
        clickedLastFrame: boolean;
        
        //
        // NOTE: Phaser Standard Methods
        //
        create() {
            
            //Local vars
            this.stage.backgroundColor = 0x222222;
            this.sfxUnlock = this.game.add.audio('unlock');
            this.sfxFootsteps = this.game.add.audio('footsteps');
            this.sfxSuccess = this.game.add.audio('success');
            
            //Map routines
            this.jsnLevel = this.add.tilemap('puzzle3jsn', 1200, 760, 1, 1);
            this.jsnLevel.createLayer('tiles').resizeWorld();
            
            this.inventory = new Inventory(this.game);
            this.createStatue();
            
            this.arrowBack = new Phaser.Sprite(this.game, 10, 80, 'arrowLeft');
            this.arrowBack.inputEnabled = true;
            this.arrowBack.scale.set(1.5);
            this.game.add.existing(this.arrowBack);
            this.arrowBack.events.onInputOver.add(this.overArrowBack, this);
            this.arrowBack.events.onInputDown.add(this.clickedArrowBack, this);
            this.arrowBack.events.onInputOut.add(this.outArrowBack, this);
            this.arrowBack.alpha = 0.5;
        }
        
        clickedArrowBack()
        {
            if(DataManager.getInstance().canClick)
            {
                this.sfxFootsteps.play();
                this.game.state.start('Room3', true, false);
            }
        }
        
        overArrowBack()
        {
            this.arrowBack.alpha = 1.0;
        }
        
        outArrowBack()
        {
            this.arrowBack.alpha = 0.5;
        }
        
        //
        // NOTE: Custom Methods
        //
        findObjectsByType( type: string ): any[] {
            
            var objs = new Array();
            
            this.jsnLevel.objects['objects'].forEach(
                function(element)
                {
                    if (element.name === type)
                        objs.push(element);
                }
            );
            
            return objs;
        }
        
        createStatue()
        {
            this.findObjectsByType('statue').forEach((element) => {
                if(!DataManager.getInstance().puzzle3Done){
                    this.statue = new Phaser.Sprite(this.game, element.x + element.width/2, element.y + element.height/2, 'statueClosedZoom');
                }
                else
                {
                    this.statue = new Phaser.Sprite(this.game, element.x + element.width/2, element.y + element.height/2, 'statueSolvedZoom');
                }
                this.game.add.existing(this.statue);
                this.statue.anchor.set(0.5);
                this.statue.inputEnabled = true;
                this.statue.events.onInputDown.add(this.handleClick, this);
            }, this);
        }
        
        update()
        {
            this.inventory.tick();
        }
        
        handleClick()
        {
            if(!DataManager.getInstance().canClick)
            {
                return;
            }
            if(!DataManager.getInstance().puzzle3Done)
            {
                if(DataManager.getInstance().vialY && DataManager.getInstance().vialC && DataManager.getInstance().vialM)
                {
                    new Monologue(this.game, "I got a piece... and it seems there's something written on the wall.");
                    this.statue.loadTexture('statueSolvedZoom');
                    this.sfxUnlock.play();
                    this.sfxSuccess.play();
                    //create piece1, interpolate to Inv
                        this.piece1 = new Phaser.Sprite(this.game, this.statue.position.x, this.statue.position.y, 'piece1puzzle64');
                        this.piece1.position.set(this.statue.position.x - this.piece1.width/2, this.statue.position.y - this.piece1.height/2);
                        this.game.add.existing(this.piece1);
                        // new Monologue(this.game, "Hmmm, what is this?");
                        DataManager.getInstance().canClick = false;
                        var tween = this.game.add.tween(this.piece1).to( { x: [ 0, this.piece1.width + 10 ], y: [ this.piece1.position.y, 10] }, 1000, "Sine.easeInOut", true, 0, 0, false);
                        tween.interpolation(Phaser.Math.bezierInterpolation);
                        tween.onComplete.add( () => {
                            DataManager.getInstance().canClick = true;
                            DataManager.getInstance().piece3 = true;
                            this.piece1.destroy();
                        });
                    
                    DataManager.getInstance().puzzle3Done = true;
                    DataManager.getInstance().vialY = false;
                    DataManager.getInstance().vialC = false;
                    DataManager.getInstance().vialM = false;
                }
                else
                {
                    new Monologue(this.game, "These vials need to be filled with something, but what?");
                }
            }
            else
            {
                new Monologue(this.game, "Hmmm, there's some kind of code in the wall..."); 
            }
        }
    }
}