/// <reference path="../../tsDefinitions/phaser.d.ts" />
/// <reference path="../DataManager.ts" />
/// <reference path="../Monologue.ts" />


//


module GGJ16 {
    
    export class Puzzle4 extends Phaser.State {
        
        //
        // NOTE: Game Attributes
        //
        sfxPressButton: Phaser.Sound;
        sfxLockerOpen: Phaser.Sound;
        jsnLevel: Phaser.Tilemap;
        inventory: Inventory;
        sfxFootsteps: Phaser.Sound;
        sfxPickPaper: Phaser.Sound;
        sfxSuccess: Phaser.Sound;
        sfxGrab: Phaser.Sound;
        sfxUnlock: Phaser.Sound;
        
        buttons: Array<Puzzle4Button>;
        sequence: Array<number>;
        nextIdx: number;
        countProgress: number;
        arrowBack: Phaser.Sprite;
        canGoBack: boolean;
        
        //
        // NOTE: Control
        //
        clickedLastFrame: boolean;
        
        
        //
        // NOTE: Phaser Standard Methods
        //
        create() {
            //Local vars
            this.stage.backgroundColor = 0x222222;
            this.canGoBack = false;
            this.clickedLastFrame = false;
            this.buttons = new Array<Puzzle4Button>();
            this.sequence = [1, 3, 5, 2, 4];
            this.countProgress = 0;
            this.nextIdx = this.sequence[this.countProgress];
            
            this.inventory = new Inventory(this.game);
            
            this.sfxFootsteps = this.game.add.audio('footsteps');
            this.sfxPickPaper = this.game.add.audio('pickpaper');
            this.sfxSuccess = this.game.add.audio('success');
            this.sfxUnlock = this.game.add.audio('unlock');
            this.sfxGrab     = this.game.add.audio('drag');
            
            //Map routines
            this.jsnLevel = this.add.tilemap('puzzle4jsn', 1200, 760, 1, 1);
            this.jsnLevel.addTilesetImage('puzzle4bg');
            this.jsnLevel.createLayer('tiles').resizeWorld();
            
            this.createButtons();
            
            // this.arrowBack = new Phaser.Sprite(this.game, this.game.world.centerX, this.game.world.height - 80, 'arrowDown');
            // this.arrowBack.inputEnabled = true;
            // this.game.add.existing(this.arrowBack);
            // this.arrowBack.events.onInputOver.add(this.overArrowBack, this);
            // this.arrowBack.events.onInputDown.add(this.clickedArrowBack, this);
            // this.arrowBack.events.onInputOut.add(this.outArrowBack, this);
            // this.arrowBack.alpha = 0.2;
            
            this.arrowBack = new Phaser.Sprite(this.game, 10, 80, 'arrowLeft');
            this.arrowBack.inputEnabled = true;
            this.arrowBack.scale.set(1.5);
            this.game.add.existing(this.arrowBack);
            this.arrowBack.events.onInputOver.add(this.overArrowBack, this);
            this.arrowBack.events.onInputDown.add(this.clickedArrowBack, this);
            this.arrowBack.events.onInputOut.add(this.outArrowBack, this);
            this.arrowBack.alpha = 0.5;
        }
        
        //
        // NOTE: Custom Methods
        //
        
        clickedArrowBack()
        {
            if(DataManager.getInstance().canClick)
            {
                this.sfxFootsteps.play();
                this.game.state.start('Room2', true, false);
            }
        }
        
        overArrowBack()
        {
            this.arrowBack.alpha = 1.0;
        }
        
        outArrowBack()
        {
            this.arrowBack.alpha = 0.2;
        }
        
        handleClick() {
            
            // if( DataManager.getInstance().piece1 &&
            //     DataManager.getInstance().piece2 &&
            //     DataManager.getInstance().piece3 &&
            //     !DataManager.getInstance().piece1Placed &&
            //     !DataManager.getInstance().piece2Placed &&
            //     !DataManager.getInstance().piece3Placed &&
            //     DataManager.getInstance().note )
            //     {
                    
            //     }
            //  else
            //  {
            //      new Monologue(this.game, "It feels like something is missing...");
            //  }
            
        }
        
        update()
        {
            this.inventory.tick();
            
            if(DataManager.getInstance().lockerOpen && !DataManager.getInstance().lockerDone)
            {
                // DataManager.getInstance().piece2 = true;
                // DataManager.getInstance().note = true;
                DataManager.getInstance().lockerDone = true;
                DataManager.getInstance().showPieceMonologue = true;
                this.sfxFootsteps.play();
                this.game.state.start('Room2', true, false);
            }
        }
        
        goBack()
        {
            debugger;
            this.canGoBack = true;
        }
        
        createButtons() {
            
            var p = this.findObjectsByType('b1')[0];
            this.buttons.push( new Puzzle4Button( this, 'ubtn', p.x, p.y, 1 ));
            var p = this.findObjectsByType('b2')[0];
            this.buttons.push( new Puzzle4Button( this, 'ubtn', p.x, p.y, 2 ));
            var p = this.findObjectsByType('b3')[0];
            this.buttons.push( new Puzzle4Button( this, 'ubtn', p.x, p.y, 3 ));
            var p = this.findObjectsByType('b4')[0];
            this.buttons.push( new Puzzle4Button( this, 'ubtn', p.x, p.y, 4 ));
            var p = this.findObjectsByType('b5')[0];
            this.buttons.push( new Puzzle4Button( this, 'ubtn', p.x, p.y, 5 ));
        }
        
        findObjectsByType( type: string ): any[] {
            
            var objs = new Array();
            
            this.jsnLevel.objects['objects'].forEach(
                function(element)
                {
                    if (element.name === type)
                        objs.push(element);
                }
            );
            
            return objs;
        }
        
        resetButtons() {
            for( var i = 0; i < this.buttons.length; i++) {
                this.buttons[i].state = 0;
                this.buttons[i].loadTexture('ubtn');
               
            }
            this.countProgress = 0;
            this.nextIdx = this.sequence[this.countProgress];
        }
    }
}