/// <reference path="../../tsDefinitions/phaser.d.ts" />
/// <reference path="../DataManager.ts" />
/// <reference path="../Puzzle5Piece.ts"/>
/// <reference path="../Monologue.ts" />

module GGJ16 {
    
    export class Puzzle5 extends Phaser.State {
        
        //
        // NOTE: Game Attributes
        //
        sfxMovePiece: Phaser.Sound;
        sfxPlacePiece: Phaser.Sound;
        sfxSuccess: Phaser.Sound;
        sfxFootsteps: Phaser.Sound;
        jsnLevel: Phaser.Tilemap;
        arrowBack: Phaser.Sprite;
        note: Phaser.Sprite;
        sfxScream: Phaser.Sound;
        
        pieces: Array<Puzzle5Piece>;
        
        numPieceDimensions:number;
        minigameCanRun:boolean;
        pieceMoving:boolean;
        
        inventory: Inventory;
        
        //
        // NOTE: Control
        //
        clickedLastFrame: boolean;
        
        
        //
        // NOTE: Phaser Standard Methods
        //
        create() {
            
            //Local vars
            this.stage.backgroundColor = 0x222222;
            this.clickedLastFrame = false;
            this.minigameCanRun = false;
            this.pieceMoving = false;
            this.pieces = new Array<Puzzle5Piece>();
            this.numPieceDimensions = 150;
            this.sfxMovePiece = this.game.add.audio('drag');
            this.sfxPlacePiece = this.game.add.audio('unlock');
            this.sfxSuccess = this.game.add.audio('success');
            this.sfxFootsteps = this.game.add.audio('footsteps');
            this.sfxScream = this.game.add.audio('scream');
            
            this.inventory = new Inventory(this.game);
            
            //Map routines
            this.jsnLevel = this.add.tilemap('puzzle5jsn', 1200, 760, 1, 1);
            this.jsnLevel.addTilesetImage('puzzle5bg');
            this.jsnLevel.createLayer('tiles').resizeWorld();
            
            // this.arrowBack = new Phaser.Sprite(this.game, this.game.world.centerX, this.game.world.height - 80, 'arrowDown');
            // this.arrowBack.inputEnabled = true;
            // this.game.add.existing(this.arrowBack);
            // this.arrowBack.events.onInputOver.add(this.overArrowBack, this);
            // this.arrowBack.events.onInputDown.add(this.clickedArrowBack, this);
            // this.arrowBack.events.onInputOut.add(this.outArrowBack, this);
            // this.arrowBack.alpha = 0.2;
            
            this.arrowBack = new Phaser.Sprite(this.game, 10, 80, 'arrowLeft');
            this.arrowBack.inputEnabled = true;
            this.arrowBack.scale.set(1.5);
            this.game.add.existing(this.arrowBack);
            this.arrowBack.events.onInputOver.add(this.overArrowBack, this);
            this.arrowBack.events.onInputDown.add(this.clickedArrowBack, this);
            this.arrowBack.events.onInputOut.add(this.outArrowBack, this);
            this.arrowBack.alpha = 0.5;
            
            this.positionPieces();
        }
        
        clickedArrowBack()
        {
            this.sfxFootsteps.play();
            this.game.state.start('Room1', true, false);
        }
        
        overArrowBack()
        {
            this.arrowBack.alpha = 1.0;
        }
        
        outArrowBack()
        {
            this.arrowBack.alpha = 0.2;
        }
        
        update() {
            
            this.inventory.tick();
            
            if( !this.pieceMoving ) {
            
                if( this.game.input.activePointer.isDown && !this.clickedLastFrame ) {
                    this.handleClick();
                    this.clickedLastFrame = true;
                }
            
                if( this.game.input.activePointer.isUp && this.clickedLastFrame ) {
                    this.clickedLastFrame = false;
                }
                
                this.checkEndGame();
                
            }
        
        }
        
        //
        // NOTE: Custom Methods
        //
        
        handleClick() {
            
            if( DataManager.getInstance().piece1 &&
                DataManager.getInstance().piece2 &&
                DataManager.getInstance().piece3 &&
                !DataManager.getInstance().piece1Placed &&
                !DataManager.getInstance().piece2Placed &&
                !DataManager.getInstance().piece3Placed &&
                DataManager.getInstance().note )
                {
                    this.game.add.tween(this.pieces[0]).to({alpha: 1.0}, 350, Phaser.Easing.Linear.None, true);
                    this.game.add.tween(this.pieces[1]).to({alpha: 1.0}, 350, Phaser.Easing.Linear.None, true);
                    this.game.add.tween(this.pieces[2]).to({alpha: 1.0}, 350, Phaser.Easing.Linear.None, true);
                    // this.pieces[0].alpha = 1.0;
                    // this.pieces[1].alpha = 1.0;
                    // this.pieces[2].alpha = 1.0;
                    this.sfxPlacePiece.play();
                    DataManager.getInstance().piece1Placed = true;
                    DataManager.getInstance().piece2Placed = true;
                    DataManager.getInstance().piece3Placed = true;
                    DataManager.getInstance().piece1 = false;
                    DataManager.getInstance().piece2 = false;
                    DataManager.getInstance().piece3 = false;
                    this.minigameCanRun = true;
                    
                    this.game.add.existing(this.note);
                    DataManager.getInstance().note = false;
                    DataManager.getInstance().notePlaced = true;
                    
                }
             else if((!DataManager.getInstance().piece1 ||
                    !DataManager.getInstance().piece2  ||
                    !DataManager.getInstance().piece3)  &&
                    (!DataManager.getInstance().note && !DataManager.getInstance().notePlaced))
             {
                 if(this.game.input.activePointer.position.y > 60)
                 {
                    new Monologue(this.game, "It feels like something is missing...");
                 }
             }
        }
        
        positionPieces() {
            
            var p = this.findObjectsByType('p1')[0];
            this.pieces.push( new Puzzle5Piece( this, 'piece1puzzle', 
                                                p.x + this.numPieceDimensions/2, 
                                                p.y + this.numPieceDimensions/2, 
                                                p.properties.xidx, 
                                                p.properties.yidx ));
            var p = this.findObjectsByType('p2')[0];
            this.pieces.push( new Puzzle5Piece( this, 'piece2puzzle', 
                                                p.x + this.numPieceDimensions/2, 
                                                p.y + this.numPieceDimensions/2, 
                                                p.properties.xidx, 
                                                p.properties.yidx ));
            var p = this.findObjectsByType('p3')[0];
            this.pieces.push( new Puzzle5Piece( this, 'piece3puzzle', 
                                                p.x + this.numPieceDimensions/2, 
                                                p.y + this.numPieceDimensions/2, 
                                                p.properties.xidx, 
                                                p.properties.yidx ));
            var p = this.findObjectsByType('p4')[0];
            this.pieces.push( new Puzzle5Piece( this, 'piece4puzzle', 
                                                p.x + this.numPieceDimensions/2, 
                                                p.y + this.numPieceDimensions/2, 
                                                p.properties.xidx, 
                                                p.properties.yidx ));
            var p = this.findObjectsByType('p5')[0];
            this.pieces.push( new Puzzle5Piece( this, 'piece5puzzle', 
                                                p.x + this.numPieceDimensions/2, 
                                                p.y + this.numPieceDimensions/2, 
                                                p.properties.xidx, 
                                                p.properties.yidx ));
            var p = this.findObjectsByType('p6')[0];
            this.pieces.push( new Puzzle5Piece( this, 'piece6puzzle', 
                                                p.x + this.numPieceDimensions/2, 
                                                p.y + this.numPieceDimensions/2, 
                                                p.properties.xidx, 
                                                p.properties.yidx ));
            var p = this.findObjectsByType('p7')[0];
            this.pieces.push( new Puzzle5Piece( this, 'piece7puzzle', 
                                                p.x + this.numPieceDimensions/2, 
                                                p.y + this.numPieceDimensions/2, 
                                                p.properties.xidx, 
                                                p.properties.yidx ));
            var p = this.findObjectsByType('p8')[0];
            this.pieces.push( new Puzzle5Piece( this, 'piece8puzzle', 
                                                p.x + this.numPieceDimensions/2, 
                                                p.y + this.numPieceDimensions/2, 
                                                p.properties.xidx, 
                                                p.properties.yidx ));

            var n = this.findObjectsByType('note')[0];
            this.note = new Phaser.Sprite(this.game, n.x + n.width/2, n.y + n.height/2, 'noteBig');
            this.note.anchor.set(0.5);
            
            if(!DataManager.getInstance().piece1Placed &&
               !DataManager.getInstance().piece2Placed &&
               !DataManager.getInstance().piece3Placed)
            {
                this.pieces[0].alpha = 0.0;
                this.pieces[1].alpha = 0.0;
                this.pieces[2].alpha = 0.0;
            }
            else
            {
                if(DataManager.getInstance().note)
                {
                    this.game.add.tween(this.pieces[0]).to({alpha: 1.0}, 350, Phaser.Easing.Linear.None, true);
                    this.game.add.tween(this.pieces[1]).to({alpha: 1.0}, 350, Phaser.Easing.Linear.None, true);
                    this.game.add.tween(this.pieces[2]).to({alpha: 1.0}, 350, Phaser.Easing.Linear.None, true);
                    DataManager.getInstance().piece1Placed = true;
                    DataManager.getInstance().piece2Placed = true;
                    DataManager.getInstance().piece3Placed = true;
                    this.sfxPlacePiece.play();
                    DataManager.getInstance().piece1 = false;
                    DataManager.getInstance().piece2 = false;
                    DataManager.getInstance().piece3 = false;
                    this.minigameCanRun = true;
                    
                    this.game.add.existing(this.note);
                    DataManager.getInstance().note = false;
                    DataManager.getInstance().notePlaced = true;
                }
                else if(DataManager.getInstance().notePlaced)
                {
                    this.game.add.tween(this.pieces[0]).to({alpha: 1.0}, 0, Phaser.Easing.Linear.None, true);
                    this.game.add.tween(this.pieces[1]).to({alpha: 1.0}, 0, Phaser.Easing.Linear.None, true);
                    this.game.add.tween(this.pieces[2]).to({alpha: 1.0}, 0, Phaser.Easing.Linear.None, true);
                    DataManager.getInstance().piece1Placed = true;
                    DataManager.getInstance().piece2Placed = true;
                    DataManager.getInstance().piece3Placed = true;
                    
                    DataManager.getInstance().piece1 = false;
                    DataManager.getInstance().piece2 = false;
                    DataManager.getInstance().piece3 = false;
                    this.minigameCanRun = true;
                    
                    this.game.add.existing(this.note);
                    DataManager.getInstance().note = false;
                    DataManager.getInstance().notePlaced = true;
                }
            }
        }
        
        checkEndGame() {
            if( Number(this.pieces[0].xidx) == 0 && Number(this.pieces[0].yidx) == 0 &&
                Number(this.pieces[1].xidx) == 1 && Number(this.pieces[1].yidx) == 0 &&
                Number(this.pieces[2].xidx) == 2 && Number(this.pieces[2].yidx) == 0 &&
                Number(this.pieces[3].xidx) == 0 && Number(this.pieces[3].yidx) == 1 &&
                Number(this.pieces[4].xidx) == 1 && Number(this.pieces[4].yidx) == 1 &&
                Number(this.pieces[5].xidx) == 2 && Number(this.pieces[5].yidx) == 1 &&
                Number(this.pieces[6].xidx) == 0 && Number(this.pieces[6].yidx) == 2 &&
                Number(this.pieces[7].xidx) == 1 && Number(this.pieces[7].yidx) == 2 )
                {
                    //
                    // TODO: End Game
                    //
                    this.pieceMoving = true;
                    this.sfxSuccess.play();
                    this.sfxScream.play();
                    this.game.state.start('Outro', true, false);
                }
        }
        
        findObjectsByType( type: string ): any[] {
            
            var objs = new Array();
            
            this.jsnLevel.objects['objects'].forEach(
                function(element)
                {
                    if (element.name === type)
                        objs.push(element);
                }
            );
            
            return objs;
        }
    }
}