/// <reference path="../../tsDefinitions/phaser.d.ts" />
/// <reference path="../../tsDefinitions/physicsType.d.ts" />
/// <reference path="../Monologue.ts" />
/// <reference path="../DataManager.ts" />


module GGJ16 {
    
    export class Room1 extends Phaser.State {
        //
        // NOTE: Game Attributes
        //
        jsnMap: Phaser.Tilemap;
        leftArrow: Phaser.Sprite;
        rightArrow: Phaser.Sprite;
        skull: Phaser.Sprite;
        bottomDrawer: Phaser.Sprite;
        topDrawer: Phaser.Sprite;
        ruby: Phaser.Sprite;
        keyDrawer: Phaser.Sprite;
        piece1: Phaser.Sprite;
        chest: Phaser.Sprite;
        finishedClickingSkull: boolean;
        finishedClickingTopDrawer: boolean;
        finishedClickingBottomDrawer: boolean;
        inventory: Inventory;
        sfxLocked:Phaser.Sound;
        sfxOpen:Phaser.Sound;
        sfxSkull:Phaser.Sound;
        sfxRuby:Phaser.Sound;
        sfxKey:Phaser.Sound;
        sfxSuccess:Phaser.Sound;
        sfxFootsteps:Phaser.Sound;
        //
        // NOTE: Phaser Standard Methods
        //
        create() {
            
            this.sfxLocked = this.game.add.audio('unlock');
            this.sfxOpen = this.game.add.audio('unlock');
            this.sfxSkull = this.game.add.audio('skullstatue');
            this.sfxRuby = this.game.add.audio('pickruby');
            this.sfxKey = this.game.add.audio('key');
            this.sfxSuccess = this.game.add.audio('success');
            this.sfxFootsteps = this.game.add.audio('footsteps', 1);
            
            this.finishedClickingSkull = this.finishedClickingTopDrawer = this.finishedClickingBottomDrawer = true;
            
            //Screen routines
            this.stage.backgroundColor = 0x222222;
            
            //Inventory
            this.inventory = new Inventory(this.game);
            
            //Init items sprites
            //Skull
            if(DataManager.getInstance().skullMouthOpen && !DataManager.getInstance().skullDone)
            {
                this.skull = new Phaser.Sprite(this.game, 0, 0, 'skullMouthOpenWithKey');
            }
            else if(DataManager.getInstance().skullDone)
            {
                this.skull = new Phaser.Sprite(this.game, 0, 0, 'skullMouthOpenNoKey');
            }
            else
            {
                this.skull = new Phaser.Sprite(this.game, 0, 0, 'skullMissingEye');
            }
            
            //Top Drawer
            if(DataManager.getInstance().topDrawerDone)
            {
                this.topDrawer = new Phaser.Sprite(this.game, 0, 0, 'topDrawerOpenNoPiece'); 
            }
            else if(DataManager.getInstance().topDrawerOpen)
            {
                this.topDrawer = new Phaser.Sprite(this.game, 0, 0, 'topDrawerOpenWithPiece');
            }
            else
            {
                this.topDrawer = new Phaser.Sprite(this.game, 0, 0, 'topDrawerClosed');
            }
            
            //Bottom Drawer
            if(DataManager.getInstance().bottomDrawerDone)
            {
                this.bottomDrawer = new Phaser.Sprite(this.game, 0, 0, 'bottomDrawerOpenNoRuby'); 
            }
            else if(DataManager.getInstance().bottomDrawerOpen)
            {
                this.bottomDrawer = new Phaser.Sprite(this.game, 0, 0, 'bottomDrawerOpenWithRuby');
            }
            else
            {
                this.bottomDrawer = new Phaser.Sprite(this.game, 0, 0, 'bottomDrawerClosed');
            }
            
            //Chest
            this.chest = new Phaser.Sprite(this.game, 0, 0, 'chest');            
            
            //Map routines
            this.jsnMap = this.add.tilemap("room1JSON", 1200, 760, 1, 1);
            this.jsnMap.addTilesetImage('room1bg');
            this.jsnMap.createLayer('tiles').resizeWorld();
            this.createObjects();
            
            
            //Init Arrows
            this.leftArrow = new Phaser.Sprite(this.game, 40, this.game.world.centerY, 'arrowLeft');
            this.game.add.existing(this.leftArrow);
            this.leftArrow.anchor.set(0.5);
            this.leftArrow.inputEnabled = true;
            
            this.rightArrow = new Phaser.Sprite(this.game, this.game.world.width - 40, this.game.world.centerY, 'arrowRight');
            this.game.add.existing(this.rightArrow);
            this.rightArrow.anchor.set(0.5);
            this.rightArrow.inputEnabled = true;
            
            if(DataManager.getInstance().shouldShowInitialMessages)
            {
                DataManager.getInstance().canClick = false;
                new IntroMessage(this.game, "What happened? Where am I? Starving might not be a good option. I must escape this place! \n I should start by clicking stuff...");
            }
        }

        update() 
        {
            //refresh inventory
            this.inventory.tick();
            
            //Show/Hide Room Change Arrows
            if(this.game.input.activePointer.position.x < 80)
            {
                this.leftArrow.alpha = 1.0;
            }
            else if(this.game.input.activePointer.position.x > this.game.world.width - 80)
            {
                this.rightArrow.alpha = 1.0;
            }
            else{
                this.leftArrow.alpha = this.rightArrow.alpha = 0.2;
            }
            
            
            //Control room change
            if(this.leftArrow.input.pointerDown(this.game.input.activePointer.id) && DataManager.getInstance().canClick)
            {
                console.log("change to room 3");
                this.game.state.start('Room3', true, false);
                this.sfxFootsteps.play();
            }
            else if(this.rightArrow.input.pointerDown(this.game.input.activePointer.id) && DataManager.getInstance().canClick)
            {
                console.log("change to room 2");
                this.game.state.start('Room2', true, false);
                this.sfxFootsteps.play();
            }
            if(this.chest.input.pointerDown(this.game.input.activePointer.id) && DataManager.getInstance().canClick)
            {
                console.log("change to room 5");
                this.game.state.start('Puzzle5', true, false);
                this.sfxFootsteps.play();
            }
            
            //Perfumarias com alpha
            if(this.chest.input.pointerOver(this.game.input.activePointer.id)){
                this.chest.alpha = 1.0;
            }
            else{
                this.chest.alpha = 0.6;
            }
            
            if(this.skull.input.pointerOver(this.game.input.activePointer.id)){
                this.skull.alpha = 1.0;
            }
            else{
                this.skull.alpha = 0.6;
            }
            
            if(this.bottomDrawer.input.pointerOver(this.game.input.activePointer.id)){
                this.bottomDrawer.alpha = 1.0;
                // this.bottomDrawer.tint = 0xffffff;
            }
            else{
                this.bottomDrawer.alpha = 0.6;
            }
          
            if(this.topDrawer.input.pointerOver(this.game.input.activePointer.id)){
                this.topDrawer.alpha = 1.0;
            }
            else{
                this.topDrawer.alpha = 0.6;
            }
            
            
            //Skull
            if(this.finishedClickingSkull)
            {
                if(this.skull.input.pointerDown(this.game.input.activePointer.id) && DataManager.getInstance().canClick)
                {
                    this.finishedClickingSkull = false;
                    if(!DataManager.getInstance().ruby) //doesn't have the ruby
                    {
                        if(!DataManager.getInstance().skullMouthOpen)
                        {
                            new Monologue(this.game, "There is something inside... But I can't take it yet.");
                        }
                        else if(!DataManager.getInstance().skullDone)
                        {
                            // console.log("Got the key");
                            this.skull.loadTexture('skullMouthOpenNoKey');
                            DataManager.getInstance().skullDone = true;
                            // DataManager.getInstance().keyDrawer = true;
                            this.sfxKey.play();
                            
                            //create keyDrawer, interpolate to Inv
                            this.keyDrawer = new Phaser.Sprite(this.game, this.skull.position.x, this.skull.position.y, 'key');
                            this.keyDrawer.position.set(this.skull.position.x - this.keyDrawer.width/2, this.skull.position.y - this.keyDrawer.height/2);
                            this.game.add.existing(this.keyDrawer);
                            new Monologue(this.game, "Hmmm, a key...");
                            DataManager.getInstance().canClick = false;
                            var tween = this.game.add.tween(this.keyDrawer).to( { x: [ 0, this.keyDrawer.width + 10 ], y: [ this.keyDrawer.position.y, 10 ] }, 1000, "Sine.easeInOut", true, 0, 0, false);
                            tween.interpolation(Phaser.Math.bezierInterpolation);
                            tween.onComplete.add( () => {
                                DataManager.getInstance().canClick = true;
                                DataManager.getInstance().keyDrawer = true;
                                this.keyDrawer.destroy();
                            });
                        }
                    }
                    else //has the ruby
                    {
                        if(!DataManager.getInstance().skullMouthOpen) // Mouth still closed
                        {
                            this.sfxSkull.play();
                            this.skull.loadTexture('skullMouthOpenWithKey');
                            DataManager.getInstance().skullMouthOpen = true;
                            DataManager.getInstance().ruby = false;
                            new Monologue(this.game, "Ahhh, got it!");
                        }
                        else{
                            new Monologue(this.game, "It's empty.");
                        }
                    }
                }
            }
            else
            {
                if(this.skull.input.pointerUp(this.game.input.activePointer.id) && DataManager.getInstance().canClick)
                {
                    this.finishedClickingSkull = true;
                }
            }
            
            //BottomDrawer
            if(this.finishedClickingBottomDrawer)
            {
                if(this.bottomDrawer.input.pointerDown(this.game.input.activePointer.id) && DataManager.getInstance().canClick)
                {
                    this.finishedClickingBottomDrawer = false;
                    if(!DataManager.getInstance().bottomDrawerOpen) //Drawer Closed
                    {
                        this.bottomDrawer.loadTexture('bottomDrawerOpenWithRuby');
                        this.bottomDrawer.position.set(this.bottomDrawer.position.x + 17, this.bottomDrawer.position.y -7);
                        DataManager.getInstance().bottomDrawerOpen = true;
                        this.sfxOpen.play();
                    }
                    else if(!DataManager.getInstance().bottomDrawerDone) //Open but not done, showing ruby
                    {
                        this.sfxRuby.play();
                        this.bottomDrawer.loadTexture('bottomDrawerOpenNoRuby');
                        DataManager.getInstance().bottomDrawerDone = true;
                        
                        //create ruby, interpolate to Inv
                        this.ruby = new Phaser.Sprite(this.game, this.bottomDrawer.position.x, this.bottomDrawer.position.y, 'ruby');
                        this.ruby.position.set(this.bottomDrawer.position.x - this.ruby.width/2, this.bottomDrawer.position.y - this.ruby.height/2);
                        this.game.add.existing(this.ruby);
                        new Monologue(this.game, "A ruby. What should I do with it?");
                        DataManager.getInstance().canClick = false;
                        var tween = this.game.add.tween(this.ruby).to( { x: [ 0, this.ruby.width + 10 ], y: [ this.ruby.position.y, 10] }, 1000, "Sine.easeInOut", true, 0, 0, false);
                        tween.interpolation(Phaser.Math.bezierInterpolation);
                        tween.onComplete.add( () => {
                            DataManager.getInstance().canClick = true;
                            DataManager.getInstance().ruby = true;
                            this.ruby.destroy();
                        });
                    }
                    else{
                        new Monologue(this.game, "It's empty...");
                    }
                }
            }
            else
            {
                if(this.bottomDrawer.input.pointerUp(this.game.input.activePointer.id) && DataManager.getInstance().canClick)
                {
                    this.finishedClickingBottomDrawer = true;
                }
            }
            
            //TopDrawer
            if(this.finishedClickingTopDrawer)
            {
                if(this.topDrawer.input.pointerDown(this.game.input.activePointer.id) && DataManager.getInstance().canClick)
                {
                    this.finishedClickingTopDrawer = false;
                    if(!DataManager.getInstance().topDrawerOpen && DataManager.getInstance().keyDrawer) //Drawer Closed
                    {
                        this.topDrawer.loadTexture('topDrawerOpenWithPiece');
                        this.topDrawer.position.set(this.topDrawer.position.x + 5, this.topDrawer.position.y + 5);
                        DataManager.getInstance().keyDrawer = false;
                        DataManager.getInstance().topDrawerOpen = true;
                        this.sfxOpen.play();
                    }
                    else if(!DataManager.getInstance().topDrawerOpen && !DataManager.getInstance().keyDrawer) //Drawer Closed
                    {
                        new Monologue(this.game, "It's locked...");
                        this.sfxOpen.play();
                    }
                    else if(!DataManager.getInstance().topDrawerDone) //Open but not done, showing piece
                    {
                        this.topDrawer.loadTexture('topDrawerOpenNoPiece');
                        DataManager.getInstance().topDrawerDone = true;
                        
                        //create piece1, interpolate to Inv
                        this.piece1 = new Phaser.Sprite(this.game, this.topDrawer.position.x, this.topDrawer.position.y, 'piece1puzzle64');
                        this.piece1.position.set(this.topDrawer.position.x - this.piece1.width/2, this.topDrawer.position.y - this.piece1.height/2);
                        this.game.add.existing(this.piece1);
                        new Monologue(this.game, "Hmmm, what is this?");
                        this.sfxSuccess.play();
                        DataManager.getInstance().canClick = false;
                        var tween = this.game.add.tween(this.piece1).to( { x: [ 0, this.piece1.width + 10 ], y: [ this.piece1.position.y, 10] }, 1000, "Sine.easeInOut", true, 0, 0, false);
                        tween.interpolation(Phaser.Math.bezierInterpolation);
                        tween.onComplete.add( () => {
                            DataManager.getInstance().canClick = true;
                            DataManager.getInstance().piece1 = true;
                            this.piece1.destroy();
                        });
                        // DataManager.getInstance().piece1 = true;
                        // new Monologue(this.game, "Hmmm, what is this?");
                    }
                    else{
                        new Monologue(this.game, "It's empty...");
                    }
                }
            }
            else
            {
                if(this.topDrawer.input.pointerUp(this.game.input.activePointer.id))
                {
                    this.finishedClickingTopDrawer = true;
                }
            }
            
        }
        
        //
        // NOTE: Custom Methods
        //
        findObjectsByType( type: string ): any[] {
            
            var objs = new Array();
            
            this.jsnMap.objects['objects'].forEach(
                function(element)
                {
                    if (element.name === type)
                        objs.push(element);
                }
            );
            
            return objs;
        }
        
        createObjects()
        {
            this.findObjectsByType('skull').forEach((element) => {
                this.skull.position.set(element.x + element.width/2, element.y + element.height/2);
                this.game.add.existing(this.skull);
                this.skull.anchor.set(0.5);
                this.skull.inputEnabled = true;
            }, this);
            
            this.findObjectsByType('topDrawer').forEach((element) => {
                this.topDrawer.position.set(element.x + element.width/2, element.y + element.height/2);
                this.game.add.existing(this.topDrawer);
                this.topDrawer.anchor.set(0.5);
                this.topDrawer.inputEnabled = true;
            }, this);
            
            this.findObjectsByType('bottomDrawer').forEach((element) => {
                this.bottomDrawer.position.set(element.x + element.width/2, element.y + element.height/2);
                this.game.add.existing(this.bottomDrawer);
                this.bottomDrawer.anchor.set(0.5);
                this.bottomDrawer.inputEnabled = true;
            }, this);
            
            this.findObjectsByType('chest').forEach((element) => {
                this.chest.position.set(element.x + element.width/2, element.y + element.height/2);
                this.game.add.existing(this.chest);
                this.chest.anchor.set(0.5);
                this.chest.inputEnabled = true;
            }, this);
        }
    }
}