/// <reference path="../../tsDefinitions/phaser.d.ts" />
/// <reference path="../../tsDefinitions/physicsType.d.ts" />
/// <reference path="../Vial.ts" />
/// <reference path="../DataManager.ts" />
/// <reference path="../Monologue.ts" />


module GGJ16 {
    
    export class Room2 extends Phaser.State {
        //
        // NOTE: Game Attributes
        //
        jsnMap: Phaser.Tilemap;
        arrayVials: Array<Vial>;
        vialBeingHeld: Vial;
        holdingVial: boolean;
        leftArrow: Phaser.Sprite;
        rightArrow: Phaser.Sprite;
        inventory: Inventory;
        sfxPickVial:Phaser.Sound;
        sfxFillVial:Phaser.Sound;
        sfxOpenLocker:Phaser.Sound;
        sfxPickPaper:Phaser.Sound;
        sfxUnlock:Phaser.Sound;
        sfxSuccess:Phaser.Sound;
        sfxFootstep:Phaser.Sound;
        locker: Phaser.Sprite;
        piece2: Phaser.Sprite;
        note: Phaser.Sprite;
                
        //parent is not used for anything in Game - it's just for syntax check purposes in the contactListener
        parent: any;
        //
        // NOTE: Phaser Standard Methods
        //
        create() {
            
            //Screen routines
            this.stage.backgroundColor = 0x222222;
            this.holdingVial = false;
            this.vialBeingHeld = null;
            
            this.inventory = new Inventory(this.game);
            
            //Sound
            this.sfxPickVial = this.game.add.audio('pickvial');
            this.sfxFillVial = this.game.add.audio('fillvial');
            this.sfxOpenLocker = this.game.add.audio('openlocker');
            this.sfxPickPaper = this.game.add.audio('pickpaper');
            this.sfxUnlock = this.game.add.audio('unlock');
            this.sfxSuccess = this.game.add.audio('success');
            this.sfxFootstep = this.game.add.audio('footsteps');
            
            //Init Arrays
            this.arrayVials = new Array<Vial>();
            
            //Map routines
            this.jsnMap = this.add.tilemap("room2JSON", 1200, 760, 1, 1);
            this.jsnMap.addTilesetImage('room2bg');
            this.jsnMap.createLayer('tiles').resizeWorld();
            this.createObjects();
            
            if(DataManager.getInstance().showPieceMonologue)
            {
                this.sfxSuccess.play();
                new Monologue(this.game, "Another piece, and a strange note...");
                DataManager.getInstance().showPieceMonologue = false;
                
                this.sfxPickPaper.play();
                this.sfxOpenLocker.play();
                
                //piece2 and note
                this.piece2 = new Phaser.Sprite(this.game, this.locker.position.x, this.locker.position.y, 'piece2puzzle64');
                this.piece2.position.set(this.locker.position.x - this.piece2.width/2, this.locker.position.y - this.piece2.height/2);
                this.game.add.existing(this.piece2);
                // new Monologue(this.game, "Hmmm, what is this?");
                DataManager.getInstance().canClick = false;
                var tween = this.game.add.tween(this.piece2).to( { x: [ 0, this.piece2.width + 10 ], y: [ this.piece2.position.y, 10] }, 1000, "Sine.easeInOut", true, 0, 0, false);
                tween.interpolation(Phaser.Math.bezierInterpolation);
                tween.onComplete.add( () => {
                    DataManager.getInstance().canClick = true;
                    DataManager.getInstance().piece2 = true;
                    this.piece2.destroy();
                    
                    
                    this.note = new Phaser.Sprite(this.game, this.locker.position.x, this.locker.position.y, 'note');
                    this.note.position.set(this.locker.position.x - this.note.width/2, this.locker.position.y - this.note.height/2);
                    this.game.add.existing(this.note);
                    // new Monologue(this.game, "Hmmm, what is this?");
                    DataManager.getInstance().canClick = false;
                    var tween = this.game.add.tween(this.note).to( { x: [ 0, this.note.width + 10 ], y: [ this.note.position.y, 10] }, 1000, "Sine.easeInOut", true, 0, 0, false);
                    tween.interpolation(Phaser.Math.bezierInterpolation);
                    tween.onComplete.add( () => {
                        DataManager.getInstance().canClick = true;
                        DataManager.getInstance().note = true;
                        this.note.destroy();
                    });
                });
                
            }
            
            
            //Init Arrows
            this.leftArrow = new Phaser.Sprite(this.game, 40, this.game.world.centerY, 'arrowLeft');
            this.game.add.existing(this.leftArrow);
            this.leftArrow.anchor.set(0.5);
            this.leftArrow.inputEnabled = true;
            
            this.rightArrow = new Phaser.Sprite(this.game, this.game.world.width - 40, this.game.world.centerY, 'arrowRight');
            this.game.add.existing(this.rightArrow);
            this.rightArrow.anchor.set(0.5);
            this.rightArrow.inputEnabled = true;
            
        }
        
        update() 
        {
            this.inventory.tick();
            
            //Show/Hide Room Change Arrows
            if(this.game.input.activePointer.position.x < 80)
            {
                this.leftArrow.alpha = 1.0;
            }
            else if(this.game.input.activePointer.position.x > this.game.world.width - 80)
            {
                this.rightArrow.alpha = 1.0;
            }
            else{
                this.leftArrow.alpha = this.rightArrow.alpha = 0.2;
            }
            
            //Control room change
            if(this.leftArrow.input.pointerDown(this.game.input.activePointer.id) && DataManager.getInstance().canClick)
            {
                this.game.state.start('Room1', true, false);
                this.sfxFootstep.play();
            }
            else if(this.rightArrow.input.pointerDown(this.game.input.activePointer.id) && DataManager.getInstance().canClick)
            {
                console.log("change to room 3");
                this.sfxFootstep.play();
                this.game.state.start('Room3', true, false);
            }
            
            //Control Vials...
            this.arrayVials.forEach((vial)=>{
               if(vial.input.pointerOver(this.game.input.activePointer.id) || vial === this.vialBeingHeld)
               {
                   vial.alpha = 1.0;
               }    
               else if(vial.colorState < 6 || vial.colorState === vialState.empty || vial.colorState === vialState.emptyUnusable)
               {
                   vial.alpha = 0.5;
               }
               
               if(vial.input.pointerDown(this.game.input.activePointer.id))
               {    
                   if(vial.colorState < 6){ //hold only 0, 1, 2, 3, 4, 5
                       this.sfxPickVial.play();
                       this.vialBeingHeld = vial;
                       this.holdingVial = true;
                   }
                   else if(vial.colorState === vialState.empty && this.holdingVial)
                   {
                       vial.setNewColor(this.vialBeingHeld.colorState);
                       this.sfxFillVial.play();
                       this.vialBeingHeld.useThePaint();
                       this.vialBeingHeld = null;
                       this.holdingVial = false;
                   }
                   else if((vial.colorState === vialState.hRedResult ||
                            vial.colorState === vialState.hGreenResult ||
                            vial.colorState === vialState.hBlueResult) && this.holdingVial)
                   {
                       if(vial.colorState === this.vialBeingHeld.colorState+3)
                       {
                           this.sfxPickVial.play();
                           new Monologue(this.game, "There's no use in using the same color twice...");
                           this.vialBeingHeld = null;
                           this.holdingVial = false;
                           return;
                       }
                       else
                       {
                           if(vial.mixColor(this.vialBeingHeld.colorState))
                           {
                                console.log("Mixed successfuly");
                                this.sfxFillVial.play();
                                this.vialBeingHeld.useThePaint();
                                this.vialBeingHeld = null;
                                this.holdingVial = false;
                           }
                       }
                   }
               }
            });
        }
        
        //
        // NOTE: Custom Methods
        //
        findObjectsByType( type: string ): any[] {
            
            var objs = new Array();
            
            this.jsnMap.objects['objects'].forEach(
                function(element)
                {
                    if (element.name === type)
                        objs.push(element);
                }
            );
            
            return objs;
        }
        
        createObjects()
        {
            this.findObjectsByType('vialR').forEach((element) => {
                this.arrayVials.push(new Vial(this.game, element.x + element.width/2, element.y + element.height/2, vialState.red, 0));          
            }, this);
            this.findObjectsByType('vialG').forEach((element) => {
                this.arrayVials.push(new Vial(this.game, element.x + element.width/2, element.y + element.height/2, vialState.green, 0));          
            }, this);
            this.findObjectsByType('vialB').forEach((element) => {
                this.arrayVials.push(new Vial(this.game, element.x + element.width/2, element.y + element.height/2, vialState.blue, 0));          
            }, this);
            this.findObjectsByType('vial1').forEach((element) => {
                this.arrayVials.push(new Vial(this.game, element.x + element.width/2, element.y + element.height/2, vialState.empty, 1));          
            }, this);
            this.findObjectsByType('vial2').forEach((element) => {
                this.arrayVials.push(new Vial(this.game, element.x + element.width/2, element.y + element.height/2, vialState.empty, 2));          
            }, this);
            this.findObjectsByType('vial3').forEach((element) => {
                this.arrayVials.push(new Vial(this.game, element.x + element.width/2, element.y + element.height/2, vialState.empty, 3));          
            }, this);
            
            
            this.findObjectsByType('locker').forEach((element) => {
                if(DataManager.getInstance().lockerDone)
                {
                    this.locker = new Phaser.Sprite(this.game, element.x + element.width/2, element.y + element.height/2, 'lockerOpen');
                }
                else
                {
                    this.locker = new Phaser.Sprite(this.game, element.x + element.width/2, element.y + element.height/2, 'locker');
                }
                this.locker.alpha = 0.7;
                this.locker.anchor.set(0.5);
                this.locker.bringToTop();
                this.game.add.existing(this.locker);
                this.locker.inputEnabled = true;
                this.locker.events.onInputDown.add(this.lockerClicked, this);
                this.locker.events.onInputOver.add(this.lockerOver, this);
                this.locker.events.onInputOut.add(this.lockerOut, this);
            }, this);
        }
        
        lockerOver()
        {
            this.locker.alpha = 1.0;
        }
        
        lockerOut()
        {
            this.locker.alpha = 0.7;
        }
        
        lockerClicked()
        {
            if(!DataManager.getInstance().lockerDone && DataManager.getInstance().canClick)
            {
                this.game.state.start('Puzzle4', true, false);
                this.sfxUnlock.play();
            }
        }
    }
}