/// <reference path="../../tsDefinitions/phaser.d.ts" />
/// <reference path="../DataManager.ts" />
/// <reference path="../Monologue.ts" />
/// <reference path="../Puzzle4Button.ts" />

module GGJ16 {
    
    export class Room3 extends Phaser.State {
        
        //
        // NOTE: Game Attributes
        //
        jsnLevel: Phaser.Tilemap;
        statue: Phaser.Sprite;
        inventory: Inventory;
        leftArrow: Phaser.Sprite;
        rightArrow: Phaser.Sprite;
        sfxFootsteps:Phaser.Sound;
        sfxUnlock: Phaser.Sound;
        
        //
        // NOTE: Control
        //
        clickedLastFrame: boolean;
        
        
        //
        // NOTE: Phaser Standard Methods
        //
        create() {
            
            //Local vars
            this.stage.backgroundColor = 0x222222;
            this.clickedLastFrame = false;
            this.sfxFootsteps = this.game.add.audio('footsteps');
            this.sfxUnlock = this.game.add.audio('unlock');
                        
            //Map routines
            this.jsnLevel = this.add.tilemap('room3jsn', 1200, 760, 1, 1);
            this.jsnLevel.addTilesetImage('room3bg');
            this.jsnLevel.createLayer('tiles').resizeWorld();
            
            
            //Init Arrows
            this.leftArrow = new Phaser.Sprite(this.game, 40, this.game.world.centerY, 'arrowLeft');
            this.game.add.existing(this.leftArrow);
            this.leftArrow.anchor.set(0.5);
            this.leftArrow.inputEnabled = true;
            
            this.rightArrow = new Phaser.Sprite(this.game, this.game.world.width - 40, this.game.world.centerY, 'arrowRight');
            this.game.add.existing(this.rightArrow);
            this.rightArrow.anchor.set(0.5);
            this.rightArrow.inputEnabled = true;
            
            this.inventory = new Inventory(this.game);
            
            var p = this.findObjectsByType('statue')[0];
            
            if( DataManager.getInstance().puzzle3Done )
                this.statue = this.game.add.sprite( p.x + p.width/2, p.y + p.height/2, 'statueSolved');
            else
                this.statue = this.game.add.sprite( p.x + p.width/2, p.y + p.height/2, 'statueClosed');
            this.statue.anchor.set(0.5);
            this.statue.alpha = 0.7;
            this.statue.inputEnabled = true;
            this.statue.events.onInputDown.add(this.handleClick, this);
            this.statue.events.onInputOver.add(this.handleOver, this);
            this.statue.events.onInputOut.add(this.handleOut, this);
        }
        
        update()
        {
            this.inventory.tick();
            
            //Show/Hide Room Change Arrows
            if(this.game.input.activePointer.position.x < 80)
            {
                this.leftArrow.alpha = 1.0;
            }
            else if(this.game.input.activePointer.position.x > this.game.world.width - 80)
            {
                this.rightArrow.alpha = 1.0;
            }
            else{
                this.leftArrow.alpha = this.rightArrow.alpha = 0.2;
            }
            
            //Control room change
            if(this.leftArrow.input.pointerDown(this.game.input.activePointer.id) && DataManager.getInstance().canClick)
            {
                console.log("change to room 2");
                this.sfxFootsteps.play();
                this.game.state.start('Room2', true, false);
            }
            else if(this.rightArrow.input.pointerDown(this.game.input.activePointer.id) && DataManager.getInstance().canClick)
            {
                console.log("change to room 1");
                this.sfxFootsteps.play();
                this.game.state.start('Room1', true, false);
            }
        }
        
        //
        // NOTE: Custom Methods
        //
        
        handleClick() {
            //
            // TODO: Go to the 3 vials puzzle
            //
            
            if(DataManager.getInstance().canClick)
            {
                this.sfxUnlock.play();
                this.game.state.start('Puzzle3', true, false);
            }
        }
        
        handleOver()
        {
            this.statue.alpha = 1.0;
        }
        handleOut()
        {
            this.statue.alpha = 0.7;
        }
        
        findObjectsByType( type: string ): any[] {
            
            var objs = new Array();
            
            this.jsnLevel.objects['objects'].forEach(
                function(element)
                {
                    if (element.name === type)
                        objs.push(element);
                }
            );
            
            return objs;
        }
    }
}